import pytest
from typing import Union

from wvalidate import Validator, ValidatorReturn, ValidatorError
from wvalidate.validators.numeric_validator import NumericValidator, NumericValidatorException

def get_error_is_not_numeric() -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError("Is not an instance of int or float.")) 

def get_error_less_than(min: Union[int, float]) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data provided is less than {min}.")) 

def get_error_greater_than(max: Union[int, float]) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data provided is greater than {max}."))

def get_error_range(min: Union[int, float], max: Union[int, float]) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data provided is not within the range of {min} to {max}.")) 


def test_numeric_validator_is_instance_validator():
	assert issubclass(NumericValidator, Validator) == True
  

@pytest.mark.parametrize("n", ["125", True, False, [1], [""]])
def test_numeric_validator_with_min_max_invalid(n):           
	with pytest.raises(NumericValidatorException):
		NumericValidator(min=n)
	
	with pytest.raises(NumericValidatorException):
		NumericValidator(max=n)
   
@pytest.mark.parametrize("min, max", [
	(3, 1.15),
	(-1, -15),
	(50.5, -1),
	(49, 48.9),
])
def test_numeric_validator_with_range_min_max_invalid(min, max):           
	with pytest.raises(NumericValidatorException):
		NumericValidator(min=min, max=max)
	


def test_numeric_validator_validate():
	assert NumericValidator().is_valid(1.0) == ValidatorReturn(True)
	assert NumericValidator().is_valid(-100) == ValidatorReturn(True)
	assert NumericValidator().is_valid(200.9) == ValidatorReturn(True)

	assert NumericValidator().is_valid("1") == get_error_is_not_numeric()
	assert NumericValidator().is_valid(None) == get_error_is_not_numeric()
	assert NumericValidator().is_valid(True) == get_error_is_not_numeric()
	assert NumericValidator().is_valid(False) == get_error_is_not_numeric()
	assert NumericValidator().is_valid([121]) == get_error_is_not_numeric()
	assert NumericValidator().is_valid({}) == get_error_is_not_numeric()

def test_numeric_validator_validate_min():
	assert NumericValidator(min=1).is_valid(1.0) == ValidatorReturn(True)
	assert NumericValidator(min=-100.5).is_valid(-100.5) == ValidatorReturn(True)
	assert NumericValidator(min=-800).is_valid(-799) == ValidatorReturn(True)
	assert NumericValidator(min=10).is_valid(200.56) == ValidatorReturn(True)

	assert NumericValidator(min=1).is_valid(0.0) == get_error_less_than(1)
	assert NumericValidator(min=-100.21).is_valid(-100.22) == get_error_less_than(-100.21)
	assert NumericValidator(min=-80).is_valid(-100) == get_error_less_than(-80)
	assert NumericValidator(min=1000.38).is_valid(200.5) == get_error_less_than(1000.38)

def test_numeric_validator_validate_max():
	assert NumericValidator(max=1).is_valid(1) == ValidatorReturn(True)
	assert NumericValidator(max=-100.0).is_valid(-100.0) == ValidatorReturn(True)
	assert NumericValidator(max=80).is_valid(-100) == ValidatorReturn(True)
	assert NumericValidator(max=10).is_valid(-200) == ValidatorReturn(True)

	assert NumericValidator(max=1).is_valid(1.1) == get_error_greater_than(1)
	assert NumericValidator(max=-100.0).is_valid(-99) == get_error_greater_than(-100.0)
	assert NumericValidator(max=-800).is_valid(-100) == get_error_greater_than(-800)
	assert NumericValidator(max=1000.23).is_valid(2000.0) == get_error_greater_than(1000.23)

def test_numeric_validator_validate_range_min_and_max():
	assert NumericValidator(min=1, max=1).is_valid(1.0) == ValidatorReturn(True)
	assert NumericValidator(min=0, max=100).is_valid(0) == ValidatorReturn(True)
	assert NumericValidator(min=0.0, max=100).is_valid(100.0) == ValidatorReturn(True)
	assert NumericValidator(min=0, max=100).is_valid(50) == ValidatorReturn(True)
	assert NumericValidator(min=-15, max=-5).is_valid(-15.0) == ValidatorReturn(True)
	assert NumericValidator(min=-15, max=-5).is_valid(-5) == ValidatorReturn(True)
	assert NumericValidator(min=-15.52, max=-5).is_valid(-10.12) == ValidatorReturn(True)

	assert NumericValidator(min=1, max=1.9).is_valid(2) == get_error_range(min=1, max=1.9)
	assert NumericValidator(min=0.9, max=1).is_valid(0.8) == get_error_range(min=0.9, max=1)
	assert NumericValidator(min=0, max=100).is_valid(-1) == get_error_range(min=0, max=100)
	assert NumericValidator(min=0, max=100.36).is_valid(101.9) == get_error_range(min=0, max=100.36)
	assert NumericValidator(min=-15, max=-5).is_valid(-156.5) == get_error_range(min=-15, max=-5)
	assert NumericValidator(min=-15.15, max=-5).is_valid(-4) == get_error_range(min=-15.15, max=-5)