import pytest

from wvalidate import Validator, ValidatorReturn, ValidatorError
from wvalidate.validators.string_validator import StringValidator, StringValidatorException

REGEX_ONLY_NUMBERS = r'^[0-9]+$'
REGEX_ONLY_LETTERS = r'^[a-zA-Z]+$'

def get_error_is_not_string() -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError("Is not an instance of str.")) 

def get_error_is_not_equal(equal: str) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data is not equal to '{equal}'.")) 

def get_error_less_than(min: int) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data provided is less than {min} characters.")) 

def get_error_greater_than(max: int) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"The data provided is greater than {max} characters."))

def get_error_range(min: int, max: int) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(f"Data provided is not within the range of {min} to {max} characters.")) 

def get_error_regex(message: str) -> ValidatorReturn:
	return ValidatorReturn(False, ValidatorError(message)) 

def test_string_validator_is_instance_validator():
	assert issubclass(StringValidator, Validator) == True

@pytest.mark.parametrize("equal", [1.0, 1, True, False, [1], [""]])
def test_string_validator_with_equal_invalid(equal):           
	with pytest.raises(StringValidatorException):
		StringValidator(equal=equal)

@pytest.mark.parametrize("n", [1.0, "125", True, False, [1], [""]])
def test_string_validator_with_min_max_invalid(n):           
	with pytest.raises(StringValidatorException):
		StringValidator(min=n)
	
	with pytest.raises(StringValidatorException):
		StringValidator(max=n)

@pytest.mark.parametrize("n", [-1, -10, -30, -42])
def test_string_validator_with_min_max_negative_integer_invalid(n):           
	with pytest.raises(StringValidatorException):
		StringValidator(min=n)
	
	with pytest.raises(StringValidatorException):
		StringValidator(max=n)
   
@pytest.mark.parametrize("min, max", [
	(3, 1),
	(-1, -15),
	(50, -1),
	(49, 48),
])
def test_string_validator_with_range_min_max_invalid(min, max):           
	with pytest.raises(StringValidatorException):
		StringValidator(min=min, max=max)


@pytest.mark.parametrize("regex", [1.0, 1, True, False, [1], [""]])
def test_string_validator_with_regex_invalid(regex):           
	with pytest.raises(StringValidatorException):
		StringValidator(regex=regex)
	
@pytest.mark.parametrize("message_error_regex", [1.0, 1, True, False, [1], [""]])
def test_string_validator_with_message_error_regex_invalid(message_error_regex):           
	with pytest.raises(StringValidatorException):
		StringValidator(
			regex=REGEX_ONLY_NUMBERS,
			message_error_invalid_regex=message_error_regex)

def test_string_validator_validate():
	assert StringValidator().is_valid("1") == ValidatorReturn(True)
	assert StringValidator().is_valid("TEST-100") == ValidatorReturn(True)
	assert StringValidator().is_valid("") == ValidatorReturn(True)

	assert StringValidator().is_valid(1) == get_error_is_not_string()
	assert StringValidator().is_valid(1.5) == get_error_is_not_string()
	assert StringValidator().is_valid(None) == get_error_is_not_string()
	assert StringValidator().is_valid(True) == get_error_is_not_string()
	assert StringValidator().is_valid(False) == get_error_is_not_string()
	assert StringValidator().is_valid([121]) == get_error_is_not_string()
	assert StringValidator().is_valid({}) == get_error_is_not_string()


def test_string_validator_validate_equal():
	assert StringValidator(equal="1").is_valid("1") == ValidatorReturn(True)
	assert StringValidator(equal="TEST-100").is_valid("TEST-100") == ValidatorReturn(True)
	assert StringValidator(equal="").is_valid("") == ValidatorReturn(True)

	assert StringValidator(equal="123456").is_valid("132456") == get_error_is_not_equal("123456")
	assert StringValidator(equal="a").is_valid("b") == get_error_is_not_equal("a")
	assert StringValidator(equal="qwe").is_valid("ewwq") == get_error_is_not_equal("qwe")
	assert StringValidator(equal="aaaaaaa").is_valid("aaaa") == get_error_is_not_equal("aaaaaaa")
	assert StringValidator(equal="1231").is_valid("False") == get_error_is_not_equal("1231")
	assert StringValidator(equal="").is_valid("[121]") == get_error_is_not_equal("")
	assert StringValidator(equal="oi").is_valid("{}") == get_error_is_not_equal("oi")

def test_string_validator_validate_min():
	assert StringValidator(min=1).is_valid("1") == ValidatorReturn(True)
	assert StringValidator(min=0).is_valid("") == ValidatorReturn(True)
	assert StringValidator(min=2).is_valid("CINCO") == ValidatorReturn(True)
	assert StringValidator(min=3).is_valid("S2S22313221321") == ValidatorReturn(True)

	assert StringValidator(min=1).is_valid("") == get_error_less_than(1)
	assert StringValidator(min=4).is_valid("101") == get_error_less_than(4)
	assert StringValidator(min=8).is_valid("1002111") == get_error_less_than(8)
	assert StringValidator(min=5).is_valid("200") == get_error_less_than(5)

def test_string_validator_validate_max():
	assert StringValidator(max=1).is_valid("1") == ValidatorReturn(True)
	assert StringValidator(max=0).is_valid("") == ValidatorReturn(True)
	assert StringValidator(max=6).is_valid("CINCO") == ValidatorReturn(True)
	assert StringValidator(max=13).is_valid("S2S22313221") == ValidatorReturn(True)

	assert StringValidator(max=0).is_valid("1") == get_error_greater_than(0)
	assert StringValidator(max=2).is_valid("101") == get_error_greater_than(2)
	assert StringValidator(max=5).is_valid("1002111") == get_error_greater_than(5)
	assert StringValidator(max=2).is_valid("200") == get_error_greater_than(2)

def test_string_validator_validate_range_min_and_max():
	assert StringValidator(min=1, max=1).is_valid("1") == ValidatorReturn(True)
	assert StringValidator(min=0, max=0).is_valid("") == ValidatorReturn(True)
	assert StringValidator(min=1, max=5).is_valid("10050") == ValidatorReturn(True)
	assert StringValidator(min=1, max=5).is_valid("100") == ValidatorReturn(True)
	assert StringValidator(min=1, max=5).is_valid("1") == ValidatorReturn(True)
	assert StringValidator(min=3, max=6).is_valid("asdfgh") == ValidatorReturn(True)
	assert StringValidator(min=3, max=6).is_valid("asdfg") == ValidatorReturn(True)
	assert StringValidator(min=3, max=6).is_valid("asd") == ValidatorReturn(True)

	assert StringValidator(min=1, max=1).is_valid("22") == get_error_range(min=1, max=1)
	assert StringValidator(min=1, max=1).is_valid("") == get_error_range(min=1, max=1)
	assert StringValidator(min=0, max=0).is_valid("1") == get_error_range(min=0, max=0)

	assert StringValidator(min=1, max=5).is_valid("") == get_error_range(min=1, max=5)
	assert StringValidator(min=1, max=5).is_valid("101132") == get_error_range(min=1, max=5)
	assert StringValidator(min=3, max=6).is_valid("56") == get_error_range(min=3, max=6)
	assert StringValidator(min=3, max=6).is_valid("qweasdz") == get_error_range(min=3, max=6)


def test_string_validator_validate_regex():
	REGEX_ONLY_NUMBERS_ERROR = "REGEX_ONLY_NUMBERS_ERROR"
	REGEX_ONLY_LETTERS_ERROR = "REGEX_ONLY_LETTERS_ERROR"

	regex_only_numbers_validator = StringValidator(
		regex=REGEX_ONLY_NUMBERS,
		message_error_invalid_regex=REGEX_ONLY_NUMBERS_ERROR
	)

	regex_only_letters_validator = StringValidator(
		regex=REGEX_ONLY_LETTERS,
		message_error_invalid_regex=REGEX_ONLY_LETTERS_ERROR
	)

	assert regex_only_numbers_validator.is_valid("1") == ValidatorReturn(True)
	assert regex_only_numbers_validator.is_valid("1231211321") == ValidatorReturn(True)
	assert regex_only_letters_validator.is_valid("a") == ValidatorReturn(True)
	assert regex_only_letters_validator.is_valid("asdagfega") == ValidatorReturn(True)

	assert regex_only_numbers_validator.is_valid("") == get_error_regex(REGEX_ONLY_NUMBERS_ERROR)
	assert regex_only_numbers_validator.is_valid("a") == get_error_regex(REGEX_ONLY_NUMBERS_ERROR)
	assert regex_only_numbers_validator.is_valid("12312das11321") == get_error_regex(REGEX_ONLY_NUMBERS_ERROR)
	assert regex_only_letters_validator.is_valid("") == get_error_regex(REGEX_ONLY_LETTERS_ERROR)
	assert regex_only_letters_validator.is_valid("1") == get_error_regex(REGEX_ONLY_LETTERS_ERROR)
	assert regex_only_letters_validator.is_valid("asda2132132gfega") == get_error_regex(REGEX_ONLY_LETTERS_ERROR)